//
//  AppPanel.java
//  
//
//  Created by David King on 3/12/10.
//	Edited by Mark Crompton and Jimmy Yih on 5/17/11
//  Copyright 2010 Miguel A. Carreira-Perpinan. All rights reserved.
//
/*

	=================================================================
	========================Resources================================
	=================================================================
I need:
JFrames:
http://www.exampledepot.com/egs/java.awt/frame_CreateFrame.html
	Why:		Forms window graphic
	Import:		java.awt.*;

Components:
http://java.sun.com/javase/7/docs/api/java/awt/Component.html
	Why:		Used as graphic component containers
	Import:		java.awt.*;

Button:
http://java.sun.com/javase/7/docs/api/java/awt/Button.html
	Why:			These buttons will start or load options

TextField:
http://java.sun.com/javase/7/docs/api/java/awt/TextField.html
	Why:			This allows user to input data and values

Scrollbar:
http://java.sun.com/javase/7/docs/api/java/awt/Scrollbar.html
	Why:			This can be used to scroll through iterations

Label:
http://java.sun.com/javase/7/docs/api/java/awt/Label.html
	Why:			This will help label buttons

Panel:
http://java.sun.com/javase/7/docs/api/java/awt/Panel.html
http://java.sun.com/docs/books/tutorial/uiswing/components/panel.html
	Why:			This is what everything will be built in

ActionListener:	
http://java.sun.com/docs/books/tutorial/uiswing/events/actionlistener.html
	Why:			This reacts when buttons or other things are clicked
	Import:			java.awt.event.*;

AdjustmentListener:	
http://java.sun.com/j2se/1.4.2/docs/api/java/awt/event/AdjustmentListener.html
	Why:			It is used for scrollbar
	Import:			java.awt.event.*;
	Note:			You would treat this class the same as ActionListener

Layout:
http://java.sun.com/docs/books/tutorial/uiswing/layout/visual.html
	Why:			This sets up how and where components will be added

GridBagLayout:
http://java.sun.com/docs/books/tutorial/uiswing/layout/gridbag.html
	Why:			This is my prefered way of creating GUIs

Issues:
ImageIO:
http://java.sun.com/docs/books/tutorial/2d/images/
	Why:		We want to import images from user
	Issue:		Need to import javax.imageio.ImageIO;

FileChooser:
http://java.sun.com/javase/7/docs/api/javax/swing/JFileChooser.html
	Why:		We want to allow the user to pick images
	Issue:		Need to import javax.swing.JFileChooser;


		=================================================================
		=======================Constraints===============================
		=================================================================
		This is to show the different parts of GridBagConstraints
		  	and explain how to use them
		  	Note: I use all default values in the example
		-----------------------------------------------------------------
		  This line creates a new GridBagConstraints, it is recommended
			to create a new constraint for each object added
		GridBagConstraints testbag = new GridNagConstraints();
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			 ___________
			|0,0|1,0|2,0|
			|___|___|___|
			|0,1|1,1|2,1|
			|___|___|___|
			gridx specifies the row and
			gridy specifies the column
			Note: Let it be known that you do not set the columns and rows
				  in a new instance of GridBagLayout, you create it through
				  the use of Constraints
		testbag.gridx = 0;
		testbag.gridy = 0;
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			|width 2|
			 ___________    __
			|	|	|	|   height 1
			|___|___|___|   __
			gridwidth specifies how many columns the component spans
			gridheight specifies how man rows the component spans
		testbag.gridwidth = 1;
		testbag.gridheight = 1;
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			fill specifies which direction the component will
			fill its given width and height
			The parameters are:
				NONE			:Fills None at all
				HORIZONTAL		:Fills Horizontally only
				VERTICAL		:Fills Vertically only
				BOTH			:Fills both Horizontally and Vertically
		testbag.fill = GridBagConstraints.NONE;
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			These two variables are used to make the component bigger
				ipadx adds that many pixels to the
					right and left of the component
				ipady adds that many pixels to the
					top and bottom of the component
		testbag.ipadx = 0;
		testbag.ipady = 0;
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			Insets are used to put space between the component and its
				surroundings
			The parameters of Insets are as follows:
				(int top,int left,int bottom, int right)
		testbag.insets = new Insets(0,0,0,0);
		-----------------------------------------------------------------


		-----------------------------------------------------------------
			anchor places the component within its cell according to these
				names:
				 ------------------------------------------------
				|FIRST_LINE_START   PAGE_START     FIRST_LINE_END|
				|                                                |
				|                                                |
				|LINE_START           CENTER             LINE_END|
				|                                                |
				|                                                |
				|LAST_LINE_START     PAGE_END       LAST_LINE_END|
				 ------------------------------------------------
		testbag.anchor = GridBagConstraints.CENTER;
		-----------------------------------------------------------------

		-----------------------------------------------------------------
			Weights are the most important part of the whole constraint
			They determine how much weight an individual component has
			and will either make or break a GUI's look
			Weights are measured from 0 to 1
			weightx is used to measure a weight of its component compared
				to all other weights in the x direction (columns)
			weighty is used to measure a weight of its component compared
				to all other weights in the y direction (rows)
		testbag.weightx = 0;
		testbag.weighty = 0;
		-----------------------------------------------------------------
		=================================================================
 */

//This import is for buttons and layouts
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Scrollbar;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.WindowAdapter;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Scanner;
import java.util.Stack;
import java.util.Vector;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import javax.imageio.ImageIO;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JApplet;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

public class AppPanel extends JApplet implements ActionListener, AdjustmentListener
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	//Reference to itself
	AppPanel app = this;

	//Finals
	final int _GBMS_ = 1;
	final int _GBMSA_ = 2;
	final int _MS1_ = 3;

	//These are our gobal varibles in our Panel
	//This is for buttons
	JButton button_go;
	JButton button_reset;

	//This is for our filechooser

	//This is for the Scrollbar
	Scrollbar scrollbar_it;

	//This is for panels
	JPanel panel_big;
	JPanel panel_title;
	JPanel panel_displays;
	JPanel panel_scroll;
	JPanel panel_vars;
	JPanel panel_sigma;
	JPanel panel_maxit;
	JPanel panel_neighbor;
	JPanel panel_scale;
	JPanel panel_input;
	JPanel storage_input;
	JPanel panel_replay;
	JPanel storage_replay;
	JPanel panel_output;
	JPanel storage_output;
	JPanel panel_text;
	JPanel panel_algo;

	//This is for image panels
	ImagePanel imgpanel_input;
	ImagePanel imgpanel_replay;
	ImagePanel imgpanel_output;

	//This is for information labels
	JLabel title_input;
	JLabel title_output;
	JLabel title_replay;
	JLabel info_input;
	JLabel info_param;
	JLabel info_output;

	//Parameter labels and textfields
	JLabel label_sigma;
	JLabel label_maxit;
	JLabel label_scale;
	JLabel label_neighbor;
	JTextField text_sigma;
	JTextField text_maxit;
	JTextField text_scale;
	JTextField text_neighbor;

	//Our input file File
	URL file_input;

	//This is our StringBuffer
	StringBuffer stringbuff_help;
	StringBuffer stringbuff_about;

	//Our MenuBar to hold our menus
	JMenuBar menubar;
	//Our Menus
	JMenu menu_file;
	JMenu menu_about;
	JMenu menu_help;

	//Our MenuItems
	//For file
	JMenu mt_load;
	//For About
	JMenuItem mt_about;
	//For Help
	JMenuItem mt_help;
	//Quit
	JMenuItem mt_quit;
	ArrayList<JMenuItem> images;
	ArrayList<URL> imageFiles;

	JComboBox combo;

	//Our Window Adaptor to close our window
	WindowAdapter windowlistener;

	//Our important values
	int viewIts;
	int width;
	int max_it;
	int state;
	double sigma;
	boolean calculated;
	boolean running;
	boolean canceled = false;
	int K;
	Vector<double [][]> Y;
	concomp c;
	long prevtime;
	double scale;
	int radius;
	int n;
	int defaultMaxIt;
	double defaultSigma;
	double defaultScale;
	int defaultRadius;
	int defaultN;
	Future<?> future;

	//This constructor creates our components
	public void construct()
	{
		//This sets our two buttons
		button_go = new JButton("GO!");
		button_reset = new JButton("Reset Defaults");

		//This creates a file
		file_input = null;

		//This set a new panel
		panel_big = new JPanel();
		panel_title = new JPanel();
		panel_scroll = new JPanel();
		panel_displays = new JPanel();
		panel_vars = new JPanel();
		panel_sigma = new JPanel();
		panel_maxit = new JPanel();
		panel_neighbor = new JPanel();
		panel_scale = new JPanel();
		panel_input = new JPanel();
		storage_input = new JPanel();
		panel_replay = new JPanel();
		storage_replay = new JPanel();
		panel_output = new JPanel();
		storage_output = new JPanel();
		panel_text = new JPanel();
		panel_algo = new JPanel();

		//This creates our MenuBar
		menubar = new JMenuBar();
		//This creates our Menu
		menu_file = new JMenu("File");
		menu_about = new JMenu("About");
		menu_help = new JMenu("Help");
		//This creates our MenuItems
		//File
		mt_load = new JMenu("Load");
		//Help
		mt_help = new JMenuItem("Help");
		//About
		mt_about = new JMenuItem("About");
		//Quit
		mt_quit = new JMenuItem("Quit");
		
		images = new ArrayList<JMenuItem>();
		imageFiles = new ArrayList<URL>();
		
		combo = new JComboBox();

		//This creates new imagepanels
		imgpanel_input = new ImagePanel();
		imgpanel_replay = new ImagePanel();
		imgpanel_output = new ImagePanel();

		//This creates a horziontal scroll bar going from 
		//0 to 100 and increments by 1
		scrollbar_it = new Scrollbar(Scrollbar.HORIZONTAL,0,0,0,max_it+1);

		/** Added to initialize Y **/
		Y = new Vector<double[][]>();

		//This creates our info labels
		title_input = new JLabel("Input", JLabel.CENTER);
		title_output = new JLabel("Output", JLabel.CENTER);
		title_replay = new JLabel("Iteration: 0", JLabel.CENTER);
		info_input = new JLabel(inputString());
		info_param = new JLabel();
		info_output = new JLabel();

		//This creates our parameter labels and textfields
		label_sigma = new JLabel("\u03C3: ", JLabel.RIGHT);
		label_maxit = new JLabel("maxit: ", JLabel.RIGHT);
		label_scale = new JLabel("scale s: ", JLabel.RIGHT);
		label_neighbor = new JLabel("radius r: ", JLabel.RIGHT);

		/** Text Fields Modified **/
		text_sigma = new JTextField();
		text_sigma.setEnabled(false);
		text_maxit = new JTextField();
		text_maxit.setEnabled(false);
		text_scale = new JTextField();
		text_scale.setEnabled(false);
		text_neighbor = new JTextField();
		text_neighbor.setEnabled(false);

		//We create a String that will read our Help.txt file
		stringbuff_help = new StringBuffer();
		stringbuff_about = new StringBuffer();
		Scanner reader = null;
		
		/** Fixed crappy try catch statement **/
		try{
			reader = new Scanner(this.getClass().getResourceAsStream("txt/Help"));

			while(reader.hasNextLine())
				stringbuff_help.
				append(reader.nextLine()).
				append(System.getProperty("line.separator"));
			reader.close();
		}
		catch(Exception e){

		}


		try{
			reader = new Scanner(this.getClass().getResourceAsStream("txt/About"));

			while(reader.hasNextLine())
				stringbuff_about.
				append(reader.nextLine()).
				append(System.getProperty("line.separator"));
			reader.close();

		}
		catch(Exception e){

		}
	}
	
	protected void loadAppletParameters() {
        //Get the applet parameters.
    	int imgNum = 0;
    	String at = getParameter("img" + imgNum);
    	while(at != null){
    		imageFiles.add(this.getClass().getResource("img/" + at));
    		images.add(new JMenuItem(at));
    		if(at.equals("masuda2_64.png"))
    			images.get(imgNum).setEnabled(false);
    		images.get(imgNum).addActionListener(app);
    		images.get(imgNum).setActionCommand("load" + imgNum);
    		imgNum++;
    		at = getParameter("img" + imgNum);
    	}
    }

	//This method initializes our panel and all objects within
	public void init()
	{
		try{
			SwingUtilities.invokeAndWait(new Runnable(){
				public void run() {
					construct();
					loadAppletParameters();
					//This sets the layout manager for our panel
					panel_big.setLayout(new BoxLayout(panel_big, BoxLayout.Y_AXIS));

					/********************************************/
					panel_title.setLayout(new GridLayout(1, 3));
					panel_title.setPreferredSize(new Dimension(256*3, 17));
					panel_title.add(title_input);
					panel_title.add(title_replay);
					panel_title.add(title_output);

					combo.addItem("MS1");
					combo.addItem("GBMS");
					combo.addItem("GBMSa");
					combo.addActionListener(app);

					panel_algo.setLayout(new BoxLayout(panel_algo, BoxLayout.X_AXIS));
					panel_algo.add(new JLabel("Algorithm: ", JLabel.RIGHT));
					panel_algo.add(combo);

					panel_scroll.setLayout(new GridLayout(1, 3));
					panel_scroll.setPreferredSize(new Dimension(256*3, 17));
					panel_scroll.add(panel_algo);
					panel_scroll.add(scrollbar_it);
					panel_scroll.add(Box.createHorizontalGlue());


					//Panel for all displays
					panel_displays.setLayout(new GridLayout(1,3));
					panel_displays.setPreferredSize(new Dimension(256*3, 256));
					panel_displays.setMinimumSize(new Dimension(256*3, 256));
					//Panels for each display
					storage_input.setLayout(new GridBagLayout());
					storage_input.add(imgpanel_input);

					/** Added GridBagLayout to match others in center-ness **/
					storage_replay.setLayout(new GridBagLayout());
					storage_replay.add(imgpanel_replay);

					storage_output.setLayout(new GridBagLayout());
					storage_output.add(imgpanel_output);

					panel_displays.add(storage_input);
					panel_displays.add(storage_replay);
					panel_displays.add(storage_output);

					panel_text.setLayout(new GridLayout(3, 1));
					panel_text.add(info_input);
					panel_text.add(info_param);
					panel_text.add(info_output);

					//Panel for all variables
					panel_vars.setLayout(new GridLayout(1,6));
					panel_vars.setPreferredSize(new Dimension(256*3, 20));
					//Panels for each variable
					panel_sigma.setLayout(new GridLayout(1,2));
					panel_sigma.add(label_sigma);
					panel_sigma.add(text_sigma);

					panel_maxit.setLayout(new GridLayout(1,2));
					panel_maxit.add(label_maxit);
					panel_maxit.add(text_maxit);

					panel_neighbor.setLayout(new GridLayout(1,2));
					panel_neighbor.add(label_neighbor);
					panel_neighbor.add(text_neighbor);

					panel_scale.setLayout(new GridLayout(1,2));
					panel_scale.add(label_scale);
					panel_scale.add(text_scale);

					panel_vars.add(button_reset);
					panel_vars.add(panel_sigma);
					panel_vars.add(panel_maxit);
					panel_vars.add(panel_neighbor);
					panel_vars.add(panel_scale);
					panel_vars.add(button_go);

					//Add all together now
					panel_big.add(panel_title);
					panel_big.add(panel_scroll);
					panel_big.add(panel_displays);
					panel_big.add(panel_text);
					panel_big.add(panel_vars);

					add(panel_big);
					/********************************************/

					//Add our actionlisteners
					button_go.addActionListener(app);
					button_reset.addActionListener(app);
					button_reset.setFont(new Font("Dialog", Font.BOLD, 11));
					scrollbar_it.addAdjustmentListener(app);

					mt_quit.addActionListener(app);
					mt_help.addActionListener(app);
					mt_about.addActionListener(app);

					//This adds Menu to our Frame
					for(int i = 0; i < images.size(); i++)
						mt_load.add(images.get(i));
//					menu_file.add(mt_load);
					menu_file.add(mt_quit);
//					menubar.add(menu_file);
					menubar.add(mt_load);
					menu_help.add(mt_help);
					menubar.add(menu_help);
					menu_about.add(mt_about);
					menubar.add(menu_about);
					app.setJMenuBar(menubar);

					/** Added **/
					button_go.setEnabled(false);
					button_reset.setEnabled(false);

					/** Load Defaults **/
					file_input = this.getClass().getResource("img/masuda2_64.png");
					load(file_input);
				}
			});
		}
		catch(Exception e){
		}
	}

	//This methods listens for actiosn from the buttons
	public void actionPerformed(ActionEvent e)
	{ 
		//code that reacts to the action...
		//e.getSource tells us what button or object was pressed
		if(e.getSource() == button_go)
		{
			if(!running)
			{
				/** MS1 **/
				if(state == _MS1_){
					/** Sigma **/
					try{
						sigma = Float.parseFloat(text_sigma.getText());
						if(sigma <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid \u03C3: values must be greater than 0",
									"Invalid \u03C3",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex){
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid \u03C3: values must be greater than 0",
								"Invalid \u03C3",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** n **/
					try{
						n = Integer.parseInt(text_maxit.getText());
						if(n <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid n: values must be greater than 0",
									"Invalid n",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid n: values must be greater than 0",
								"Invalid n",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** Radius **/
					try{
						radius = Integer.parseInt(text_neighbor.getText());
						if(radius < 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid Radius: values must be greater than or equal to 0",
									"Invalid Radius",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid Radius: values must be greater than or equal to 0",
								"Invalid Radius",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** Scale **/
					try{
						scale = Float.parseFloat(text_scale.getText());
						if(scale <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid Scale: values must be greater than 0",
									"Invalid Scale",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid Scale: values must be greater than 0",
								"Invalid Scale",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					storage_replay.remove(imgpanel_replay);
					double zoom = 256.0 / Math.max(imgpanel_input.getMSWidth(), imgpanel_input.getMSHeight());
					imgpanel_replay = new ImagePanel(file_input,zoom,true);
					imgpanel_replay.validate();
					storage_replay.add(imgpanel_replay);
					app.validate();
				}
				/** GBMS **/
				else if(state == _GBMS_){
					/** Sigma **/
					try{
						sigma = Float.parseFloat(text_sigma.getText());
						if(sigma <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid \u03C3: values must be greater than 0",
									"Invalid \u03C3",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex){
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid \u03C3: values must be greater than 0",
								"Invalid \u03C3",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** n **/
					try{
						max_it = Integer.parseInt(text_maxit.getText());
						if(max_it <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid MaxIt: values must be greater than 0",
									"Invalid MaxIt",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid MaxIt: values must be greater than 0",
								"Invalid MaxIt",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** Scale **/
					try{
						scale = Float.parseFloat(text_scale.getText());
						if(scale <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid Scale: values must be greater than 0",
									"Invalid Scale",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid Scale: values must be greater than 0",
								"Invalid Scale",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** Radius **/
					try{
						radius = Integer.parseInt(text_neighbor.getText());
						if(radius < 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid Radius: values must be greater than or equal to 0",
									"Invalid Radius",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid Radius: values must be greater than or equal to 0",
								"Invalid Radius",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}
				}
				/** GBMSa **/
				else if(state == _GBMSA_){
					/** Sigma **/
					try{
						sigma = Float.parseFloat(text_sigma.getText());
						if(sigma <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid \u03C3: values must be greater than 0",
									"Invalid \u03C3",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex){
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid \u03C3: values must be greater than 0",
								"Invalid \u03C3",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** MaxIt **/
					try{
						max_it = Integer.parseInt(text_maxit.getText());
						if(max_it <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid MaxIt: values must be greater than 0",
									"Invalid MaxIt",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid MaxIt: values must be greater than 0",
								"Invalid MaxIt",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}

					/** Scale **/
					try{
						scale = Float.parseFloat(text_scale.getText());
						if(scale <= 0){
							JOptionPane.showMessageDialog(
									new JFrame(),
									"Invalid Scale: values must be greater than 0",
									"Invalid Scale",
									JOptionPane.WARNING_MESSAGE
							);
							return;
						}
					} catch(Exception ex) {
						JOptionPane.showMessageDialog(
								new JFrame(),
								"Invalid Scale: values must be greater than 0",
								"Invalid Scale",
								JOptionPane.WARNING_MESSAGE
						);
						return;
					}
				}
				running = true;
				prevtime = Calendar.getInstance().getTimeInMillis();

				//Disable all objects that have an listener
				mt_load.setEnabled(false);
				text_sigma.setEnabled(false);
				text_scale.setEnabled(false);
				text_neighbor.setEnabled(false);
				text_maxit.setEnabled(false);
				scrollbar_it.setEnabled(false);
				combo.setEnabled(false);
				button_go.setText("Cancel");
				button_reset.setEnabled(false);
				title_replay.setText("Iteration: 0");
				info_param.setText("");
				info_output.setText("");

				storage_output.remove(imgpanel_output);
				imgpanel_output = new ImagePanel();
				imgpanel_output.validate();
				storage_output.add(imgpanel_output);
				app.validate();

				Runnable worker;
				ExecutorService executor = Executors.newFixedThreadPool(1);

				switch(state)
				{
				case _GBMS_:
					worker = new gbms();
					future = executor.submit(worker);
					break;
				case _GBMSA_:
					worker = new agbms();
					future = executor.submit(worker);
					break;
				case _MS1_:
					worker = new ms1();
					future = executor.submit(worker);
					break;
				}
			}
			else
			{
				//Enable all objects that have an listener
				title_replay.repaint();
				app.repaint();
				button_go.setText("Canceling");
				button_go.setEnabled(false);
				canceled = true;
				app.validate();
			}
		}
		else if(e.getActionCommand().contains("load")){
			int num = Integer.decode(e.getActionCommand().substring(4));
			for(int i = 0; i < images.size(); i++)
				images.get(i).setEnabled(true);
			images.get(num).setEnabled(false);
			file_input = imageFiles.get(num);
			load(imageFiles.get(num));
		}
		else if(e.getSource() == button_reset){
			text_sigma.setText("" + defaultSigma);
			text_scale.setText("" + defaultScale);
			text_neighbor.setText("" + defaultRadius);
			if(state == _MS1_)
				text_maxit.setText("" + defaultN);
			else
				text_maxit.setText("" + defaultMaxIt);
		}
		else if(e.getSource() == combo)
		{
			if(combo.getSelectedItem().equals("GBMS"))
			{
				state = _GBMS_;
				text_neighbor.setEnabled(true);
				text_maxit.setText(""+max_it);
				label_maxit.setText("maxit: ");
				button_go.setEnabled(true);
				text_sigma.setEnabled(true);
				text_maxit.setEnabled(true);
				text_neighbor.setEnabled(true);
				text_scale.setEnabled(true);
				title_replay.setText("Iteration: 0");
				scrollbar_it.setEnabled(true);
				scrollbar_it.setVisible(true);
			}
			else if(combo.getSelectedItem().equals("GBMSa"))
			{
				state = _GBMSA_;
				text_neighbor.setEnabled(false);
				text_maxit.setText(""+max_it);
				label_maxit.setText("maxit: ");
				button_go.setEnabled(true);
				text_sigma.setEnabled(true);
				text_maxit.setEnabled(true);
				text_scale.setEnabled(true);
				title_replay.setText("Iteration: 0");
				scrollbar_it.setEnabled(true);
				scrollbar_it.setVisible(true);
			}
			else if(combo.getSelectedItem().equals("MS1"))
			{
				state = _MS1_;
				text_neighbor.setEnabled(true);
				text_maxit.setText(""+n);
				label_maxit.setText("n: ");
				button_go.setEnabled(true);
				text_sigma.setEnabled(true);
				text_maxit.setEnabled(true);
				text_neighbor.setEnabled(true);
				text_scale.setEnabled(true);
				title_replay.setText("Percent: 0");
				scrollbar_it.setEnabled(false);
				scrollbar_it.setVisible(false);
			}
		}
		else if(e.getSource() == mt_about)
		{
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					JOptionPane.showMessageDialog(
							new JFrame(),
							stringbuff_about,
							"About",
							JOptionPane.INFORMATION_MESSAGE
					);
				}
			});
		}
		else if(e.getSource() == mt_help)
		{
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					JOptionPane.showMessageDialog(
							new JFrame(),
							stringbuff_help,
							"Help",
							JOptionPane.QUESTION_MESSAGE
					);
				}
			});
		}
	}

	public void load(URL file){
		BufferedImage img;
		try {
			img = ImageIO.read(file);
		}
		catch (IOException ex) {img = null;}
		if(img.getHeight() > 256 || img.getWidth() > 256){
			JOptionPane.showMessageDialog(
					app,
					"Image size is too large, must be less than 256x256 pixels.",
					"Invalid Size",
					JOptionPane.WARNING_MESSAGE
			);
			return;
		}
		switch(img.getType()){
		case BufferedImage.TYPE_CUSTOM :
		case BufferedImage.TYPE_USHORT_555_RGB :
		case BufferedImage.TYPE_USHORT_565_RGB :
		case BufferedImage.TYPE_USHORT_GRAY :
			JOptionPane.showMessageDialog(
					app,
					"The image data is of an unknown configuration.",
					"Invalid Image",
					JOptionPane.WARNING_MESSAGE
			);
			return;
		}

		storage_input.remove(imgpanel_input);
		storage_replay.remove(imgpanel_replay);
		storage_output.remove(imgpanel_output);
		storage_input.invalidate();
		storage_replay.invalidate();
		storage_output.invalidate();
		storage_input.validate();
		storage_replay.validate();
		storage_output.validate();
		app.validate();

		imgpanel_input = new ImagePanel(img,1);
		double zoom = 256.0 / Math.max(imgpanel_input.getMSWidth(), imgpanel_input.getMSHeight());
		imgpanel_replay = new ImagePanel(img,zoom);
		imgpanel_output = new ImagePanel();
		storage_input.add(imgpanel_input);
		storage_replay.add(imgpanel_replay);
		storage_output.add(imgpanel_output);
		storage_input.repaint();
		storage_replay.repaint();
		storage_output.repaint();
		storage_input.validate();
		storage_replay.validate();
		storage_output.validate();
		app.repaint();

		info_input.setText(inputString());
		info_param.setText("");
		info_output.setText("");
		app.validate();
		//Load Defaults for loaded image
		scale = Math.max(imgpanel_input.getMSWidth(),
				imgpanel_input.getMSHeight()) * 2;
		sigma = Math.min(imgpanel_input.getMSWidth(),
				imgpanel_input.getMSHeight())/6.0f;
		sigma = (int)(sigma*100.0) / 100.0;
		max_it = 50;
		n = 3;
		radius = 0;
		defaultMaxIt = max_it;
		defaultSigma = sigma;
		defaultScale = scale;
		defaultRadius = radius;
		defaultN = n;
		text_sigma.setText("" + sigma);
		text_scale.setText("" + scale);
		text_neighbor.setText("" + radius);
		text_maxit.setText("" + max_it);
		state = 0;
		button_go.setEnabled(false);

		button_reset.setEnabled(true);
		state = 0;
		combo.setSelectedIndex(0);

		state = _MS1_;
		combo.setSelectedIndex(0);
		text_neighbor.setEnabled(true);
		text_maxit.setText(""+n);
		label_maxit.setText("n: ");
		button_go.setEnabled(true);
		text_sigma.setEnabled(true);
		text_maxit.setEnabled(true);
		text_neighbor.setEnabled(true);
		text_scale.setEnabled(true);
		title_replay.setText("Percent: 0");
		scrollbar_it.setEnabled(false);
		scrollbar_it.setVisible(false);
		scrollbar_it.setValue(0);
		scrollbar_it.setMaximum(1);
	}

	//This is used for seeing the actions of the scrollbar
	public void adjustmentValueChanged(AdjustmentEvent e)
	{
		//This code reacts to scrollbar adjustments
		//e.getSource() tells us which scrollbar is changed
		if(e.getSource() == scrollbar_it)
			//This takes the value of the current scrollbar and puts
			//it as text in the appropriate label
		{
			viewIts = scrollbar_it.getValue();
			if(calculated)
			{
				title_replay.setText("Iteration: " + viewIts);
				storage_replay.remove(imgpanel_replay);
				double zoom = 256.0 / Math.max(imgpanel_input.getMSWidth(), imgpanel_input.getMSHeight());
				imgpanel_replay = new ImagePanel(
						new CreateImage(
								Y.get(viewIts),
								imgpanel_input.getMSWidth(),
								imgpanel_input.getMSHeight(),
								scale
						).getImage(), zoom
				);
				imgpanel_replay.validate();
				storage_replay.add(imgpanel_replay);
				app.validate();
			}
		}
	}

	//This creates the string that inputs how long a process took
	private String timeDiff()
	{
		long timediff = Calendar.getInstance().getTimeInMillis() - prevtime;
		String output = "";
		double diff = timediff/10.0;
		diff = (int)(Math.round(diff)) / 100.0;
		output = diff + "s " + output;
		return output;
	}
	
	public String paramString(){
		String output = "Algorithm: ";
		switch(state)
		{
		case _GBMS_:
			output += "GBMS ";
			break;
		case _GBMSA_:
			output += "GBMSa ";
			break;
		case _MS1_:
			output += "MS1 ";
		}
		double printSigma = (int)(Math.round(sigma*100.0)) / 100.0;
		output += "(\u03c3=" + printSigma + ", ";
		if((_GBMS_ == state || _MS1_ == state) && radius != 0)
			output += "r="+radius+", ";
		if(state == _MS1_)
			output += "n="+n+", ";
		output += "s="+scale+").";
		return output;
	}

	//This creates the string that displays the information on the output
	private String outputString(int Its)
	{
		String output = "";
		output += "Result: ";

		if(state != _MS1_)
			output += Its + " Iterations, ";
		output += timeDiff() + "run time, ";
		output += K +" Clusters.";
		return output;
	}

	private String inputString()
	{
		String output;
		if(file_input != null){
			output = "Image: ";
			String fileName = file_input.getFile();
			fileName = fileName.substring(fileName.lastIndexOf('/')+1);
			output += fileName + ", " +
			imgpanel_input.getMSWidth() + "x" +
			imgpanel_input.getMSHeight() + ", ";
			String type = "";
			switch(imgpanel_input.getImage().getType()){
			case BufferedImage.TYPE_4BYTE_ABGR :
			case BufferedImage.TYPE_4BYTE_ABGR_PRE :
			case BufferedImage.TYPE_INT_ARGB : 
			case BufferedImage.TYPE_INT_ARGB_PRE :
				type = " RGBA.";
				break;
			case BufferedImage.TYPE_3BYTE_BGR :
			case BufferedImage.TYPE_INT_BGR :
			case BufferedImage.TYPE_INT_RGB :
			case BufferedImage.TYPE_BYTE_BINARY :
			case BufferedImage.TYPE_BYTE_INDEXED :
				type = " RGB.";
				break;
			case BufferedImage.TYPE_BYTE_GRAY :
				type = " Grayscale.";
				break;
			}
			output += type;
		}
		else
			output = "No input file.";
		return output;
	}

	private void enablePushables(){
		mt_load.setEnabled(true);
		if(!button_go.getText().equals("Canceling"))
			scrollbar_it.setEnabled(true);
		button_go.setText("GO!");
		button_go.setEnabled(true);
		button_reset.setEnabled(true);
		text_sigma.setEnabled(true);
		text_scale.setEnabled(true);
		if(state != _GBMSA_)
			text_neighbor.setEnabled(true);
		text_maxit.setEnabled(true);
		combo.setEnabled(true);
		running = false;
		calculated = true;
		button_go.setEnabled(true);
	}

	//This creates a struct like object to run gbms
	private class gbms implements Runnable
	{
		public void run()
		{
			try
			{
				Y.clear();
				viewIts = 0;
				//Begin gbms
				//Default values
				double [][] X = imgpanel_input.getMSPixels(scale);
				int N = X.length,DD = X[0].length,Its = 0;
				double tol = 1E-3f;
				double min_diff = .01f;
				boolean convcrit = max_it >= 1;
				int height = imgpanel_input.getMSHeight();
				int width = imgpanel_input.getMSWidth();

				//Ending condition variables
				double [] Xmov = new double[N];
				int Nbins = (int) Math.floor(0.9*N);
				double [] hist = new double[Nbins];
				double oldH = -1000;

				Y.add(new double[N][DD]);
				for(int i=0;i<N;i++)
					for(int j=0;j<DD;j++)
						Y.get(Its)[i][j] = X[i][j];

				storage_replay.remove(imgpanel_replay);
				double zoom = 256.0 / Math.max(imgpanel_input.getMSWidth(), imgpanel_input.getMSHeight());
				imgpanel_replay = new ImagePanel(
						new CreateImage(
								Y.get(viewIts),
								imgpanel_input.getMSWidth(),
								imgpanel_input.getMSHeight(),
								scale
						).getImage(),zoom
				);
				imgpanel_replay.validate();
				storage_replay.add(imgpanel_replay);
				app.validate();
				while(convcrit)
				{
					Y.add(new double[N][DD]);
					for(int i=0;i<N;i++)
					{
						if(radius==0)
						{
							double D = 0;
							double [] W = new double[N];
							for(int j=0;j<N;j++)
							{
								W[j] = 0;
								for(int k=0;k<DD;k++)
								{
									if(canceled){
										enablePushables();
										button_go.setEnabled(true);
										canceled = false;
										return;
									}
									W[j] = W[j] + (double)Math.pow((X[i][k]-
											X[j][k]),
											2);
								}
								W[j] = (double)Math.exp(-1*W[j]/(2*sigma*sigma));
								D = D+W[j];
							}
							for(int j=0;j<DD;j++)
							{
								Y.get(Its+1)[i][j] = 0;
								for(int k=0;k<N;k++)
									Y.get(Its+1)[i][j] =
										Y.get(Its+1)[i][j]+W[k]*X[k][j]/D;
							}
						}
						else
						{
							double D = 0;
							int xi = (int)Y.get(0)[i][0];
							int yi = (int)Y.get(0)[i][1];
							int lower = Math.max(0,yi-(radius - 1));
							int upper = Math.min(height - 1,yi+(radius - 1));
							int left = Math.max(0,xi-(radius - 1));
							int right = Math.min(width - 1,xi+(radius - 1));
							double [] W = new double[(upper-lower+1)*(right-left+1)];
							int wHigh = 0;
							for(int j=lower;j<=upper;j++)
							{
								for(int k=left;k<=right;k++)
								{
									int w_index = (j-lower)*(right - left + 1)+(k-left);
									if(w_index > wHigh)
										wHigh = w_index;
									for(int l=0;l<DD;l++)
									{
										if(canceled){
											enablePushables();
											button_go.setEnabled(true);
											canceled = false;
											return;
										}
										W[w_index] +=
											(double)
											Math.pow(X[i][l]-X[k+width*j][l],2);
									}
									W[w_index] =
										(double)
										Math.exp(-1*W[w_index]/(2*sigma*sigma));
									D+=W[w_index];
								}
							}
							for(int j=0;j<DD;j++)
							{
								Y.get(Its+1)[i][j] = 0;
								for(int l=lower;l<=upper;l++)
									for(int k=left;k<=right;k++)
									{
										int w_index=(l-lower)*(right-left + 1)+
										(k-left);
										Y.get(Its+1)[i][j]=
											Y.get(Its+1)[i][j]+
											W[w_index]*X[k+width*l][j]/D;
									}
							}
						}
					}
					Its++;
					for(int i=0;i<N;i++)
						for(int j=0;j<DD;j++)
							X[i][j] = Y.get(Its)[i][j];


					//End Conditions
					double mean = 0;
					//X is the latest set of points and updated is the orignal
					for(int i=0;i<N;i++)
					{
						Xmov[i] = 0;
						for(int j=0;j<DD;j++)
							Xmov[i] += Math.pow(Y.get(Its-1)[i][j] -
									Y.get(Its)[i][j],
									2);
						Xmov[i] = (double) Math.sqrt(Xmov[i]);
						mean += Xmov[i];
					}
					mean = mean/N;

					//Histogram portion here
					double max=Xmov[0],min=Xmov[0];
					for(int i=0;i<N;i++)
					{
						if(Xmov[i] > max)
							max = Xmov[i];
						if(Xmov[i] < min)
							min = Xmov[i];
					}
					double diff = max - min;
					double H = 0;
					for(int i=0;i<Nbins;i++)
						hist[i] = 0;
					for(int i=0;i<N;i++){
						int binIndex = (int)((Xmov[i]-min)/diff*(Nbins));
						if(binIndex == Nbins)
							binIndex = Nbins-1;
						hist[binIndex]++;
					}
					for(int i=0;i<Nbins;i++)
						hist[i] = hist[i]/N;
					for(int i=0;i<Nbins;i++){
						if(hist[i] > 0){
							H += -hist[i]*Math.log(hist[i]);
						}
					}
					convcrit = (Its < max_it) && (mean > tol) && (Math.abs(oldH-H) > 1E-8);

					oldH = H;

					//update the replay
					title_replay.setText("Iteration: " + Its);
					storage_replay.remove(imgpanel_replay);
					imgpanel_replay = new ImagePanel(
							new CreateImage(
									Y.get(++viewIts),
									imgpanel_input.getMSWidth(),
									imgpanel_input.getMSHeight(),
									scale
							).getImage(),zoom
					);
					imgpanel_replay.validate();
					storage_replay.add(imgpanel_replay);
					app.validate();
				}

				c = new concomp(X,min_diff);
				K = c.K;
				//Place it in
				storage_output.remove(imgpanel_output);
				imgpanel_output = new ImagePanel(
						new CreateImage(
								c,
								Y.get(0),
								imgpanel_input.getMSWidth(),
								imgpanel_input.getMSHeight()).getImage(),1);
				imgpanel_output.validate();
				storage_output.add(imgpanel_output);
				app.validate();

				//Change scroll bar
				scrollbar_it.setMaximum(Its+1);
				scrollbar_it.setValue(viewIts);
				info_input.setText(inputString());
				info_param.setText(paramString());
				info_output.setText(outputString(Its));
				app.validate();

				/** Re-enable buttons and menu options **/
				enablePushables();
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
	}

	//This creates a struct like object to run gbmsa once bugs are out
	//until then it will just be a method
	private class agbms implements Runnable
	{
		public void run()
		{
			Y.clear();
			viewIts = 0;
			//Begin gbms
			//Default values
			double [][] X = imgpanel_input.getMSPixels(scale);
			int N = X.length,DD = X[0].length,Its = 0;
			double tol = 1E-3f;
			double min_diff = .01f;
			boolean convcrit = max_it >= 1;

			//Values used for accelerated purposes
			double [] Pi = new double [N];
			int [] Labels = new int[N];
			for(int i=0;i<N;i++)
			{
				Pi[i] = 1.0f/(double)N;
				Labels[i] = i;
			}
			int NN = N;
			boolean reduced = false;

			//Ending condition variables
			double [] Xmov = new double[N];
			int Nbins = (int) Math.floor(0.9*N);
			double [] hist = new double[Nbins];
			double oldH = -1000;

			Y.add(new double[N][DD]);
			for(int i=0;i<N;i++)
				for(int j=0;j<DD;j++)
					Y.get(Its)[i][j] = X[i][j];

			//Show starting image
			title_replay.setText("Iteration: " + Its);
			storage_replay.remove(imgpanel_replay);
			double zoom = 256.0 / Math.max(imgpanel_input.getMSWidth(), imgpanel_input.getMSHeight());
			imgpanel_replay = new ImagePanel(
					new CreateImage(
							Y.get(viewIts),
							imgpanel_input.getMSWidth(),
							imgpanel_input.getMSHeight(),
							scale
					).getImage(),zoom
			);
			imgpanel_replay.validate();
			storage_replay.add(imgpanel_replay);
			app.validate();
			int skipIts = 4;
			while(convcrit)
			{
				Y.add(new double[NN][DD]);
				/** Acceleration **/
				if(Its >= skipIts){
					c = new concomp(X,min_diff);
					double [][] X0 = new double[X.length][DD];
					for(int i=0;i<X.length;i++)
						for(int j=0;j<DD;j++)
							X0[i][j] = X[i][j];
					double [] Pi0 = new double[Pi.length];
					for(int i=0;i<Pi.length;i++)
						Pi0[i] = Pi[i];
					X = new double[c.K][DD];
					for(int i=0;i<X.length;i++)
						for(int j=0;j<X[i].length;j++)
							X[i][j] = 0.0f;
					Pi = new double[c.K];
					for(int i=0;i<Pi.length;i++)
						Pi[i] = 0.0f;
					int [] Xnum = new int[X.length];
					for(int i=0;i<X0.length;i++)
					{
						for(int j=0;j<DD;j++)
							X[c.compl[i]][j] += X0[i][j];
						Xnum[c.compl[i]]++;
					}
					for(int i=0;i<X.length;i++)
						for(int j=0;j<DD;j++)
							X[i][j] /= (double)Xnum[i];
					double sum = 0.0f;
					for(int i=0;i<Pi0.length;i++)
					{
						Pi[c.compl[i]] += Pi0[i];
						sum += Pi0[i];
					}
					for(int i=0;i<Pi.length;i++)
						Pi[i] /= sum;
					for(int i=0;i<NN;i++)
						Labels[i] = c.compl[Labels[i]];
					N = Pi.length;
					reduced = true;
				}
				//Normal GBMS
				for(int i=0;i<N;i++)
				{
					double D = 0;
					double [] W = new double[N];
					for(int j=0;j<N;j++)
					{
						W[j] = 0;
						for(int k=0;k<DD;k++)
						{
							if(canceled){
								enablePushables();
								button_go.setEnabled(true);
								canceled = false;
								return;
							}
							W[j] = W[j] + (double)Math.pow((X[i][k]-X[j][k]),2);
						}
						W[j] = (double)Math.exp(-1*W[j]/(2*sigma*sigma));
						//A simple check to see if anything is reduced
						if(reduced)
							W[j] *= Pi[j];

						D = D+W[j];
					}
					for(int j=0;j<DD;j++)
					{
						//We can do this for the reduced because
						//N will never be greater than NN
						Y.get(Its+1)[i][j] = 0;
						for(int k=0;k<N;k++)
							Y.get(Its+1)[i][j] += W[k]*X[k][j]/D;
					}

				}
				Its++;
				for(int i=0;i<N;i++)
					for(int j=0;j<DD;j++)
						X[i][j] = Y.get(Its)[i][j];

				//After moving the actual X values to X, we can now
				//move those over for the expanded version of Y
				for(int i=0;i<NN;i++)
					for(int j=0;j<DD;j++)
						Y.get(Its)[i][j] = X[Labels[i]][j];

				//End Conditions
				double mean = 0;
				//X is the latest set of points and updated is the orignal
				//make sure to use NN instead of N, because we want the full set
				for(int i=0;i<NN;i++)
				{
					Xmov[i] = 0;
					for(int j=0;j<DD;j++)
						Xmov[i]+=Math.pow(Y.get(Its-1)[i][j]-Y.get(Its)[i][j],
								2);
					Xmov[i] = (double) Math.sqrt(Xmov[i]);
					mean += Xmov[i];
				}
				mean = mean/NN;

				//Histogram portion here
				double max=Xmov[0],min=Xmov[0];
				for(int i=0;i<NN;i++)
				{
					if(Xmov[i] > max)
						max = Xmov[i];
					if(Xmov[i] < min)
						min = Xmov[i];
				}
				double diff = max - min;
				double H = 0;
				for(int i=0;i<Nbins;i++)
					hist[i]=0;
				for(int i=0;i<NN;i++){
					int binIndex = (int)Math.floor((Xmov[i]-min)/diff*(Nbins));
					if(binIndex == Nbins)
						binIndex--;
					hist[binIndex]++;
				}
				for(int i=0;i<Nbins;i++)
					hist[i] = hist[i]/NN;
				for(int i=0;i<Nbins;i++)
					if(hist[i] > 0)
						H += -hist[i]*Math.log(hist[i]);
				convcrit= Its < max_it && mean > tol && Math.abs(oldH-H) > 1E-8;

				oldH = H;

				//update the replay
				title_replay.setText("Iteration: " + Its);
				storage_replay.remove(imgpanel_replay);
				imgpanel_replay = new ImagePanel(
						new CreateImage(
								Y.get(++viewIts),
								imgpanel_input.getMSWidth(),
								imgpanel_input.getMSHeight(),
								scale
						).getImage(),zoom
				);
				imgpanel_replay.validate();
				storage_replay.add(imgpanel_replay);
				app.validate();
			}

			c = new concomp(X,min_diff);
			//Must update Labels to new ones and as well update our clust object
			for(int i=0;i<NN;i++)
				Labels[i] = c.compl[Labels[i]];
			c.setLabels(Labels);
			K = c.K;
			//Place it in
			storage_output.remove(imgpanel_output);
			imgpanel_output = new ImagePanel(
					new CreateImage(
							c,
							Y.get(0),
							imgpanel_input.getMSWidth(),
							imgpanel_input.getMSHeight()
					).getImage(),1
			);
			imgpanel_output.validate();
			storage_output.add(imgpanel_output);
			app.validate();

			//Change scroll bar
			scrollbar_it.setMaximum(Its+1);
			scrollbar_it.setValue(viewIts);
			info_input.setText(inputString());
			info_param.setText(paramString());
			info_output.setText(outputString(Its));
			app.validate();

			/** Re-enable buttons and menu options **/
			enablePushables();
		}
	}


	private class ms1 implements Runnable
	{
		public void run()
		{
			double [][] X = imgpanel_input.getMSPixels(scale);
			ArrayList<double[]> colors = new ArrayList<double[]>();
			double tol = 1E-3f;
			int height = imgpanel_input.getMSHeight();
			int width = imgpanel_input.getMSWidth();
			int N = X.length,label = 1,DD = X[0].length;
			boolean convcrit = true;
			int [][][][] labeltable = new int[width][height][n][n];
			int [] Labels = new int[N];
			Stack<int []> path = new Stack<int []>();
			for(int i=0;i<N;i++)
			{
				title_replay.setText("Percent: " + ((i+1)*100/N));
				path.clear();
				double [] cur = new double[DD];
				for(int j=0;j<DD;j++)
					cur[j] = X[i][j];
				double [] prev = new double[DD];
				convcrit = true;
				//have initial case (see if visited)
				iteration:
					while(convcrit)
					{
						for(int j=0;j<DD;j++)
							prev[j] = cur[j];
						if(radius == 0){
							double D = 0;
							double [] W = new double[N];
							for(int j=0;j<N;j++)
							{
								W[j] = 0;
								for(int k=0;k<DD;k++)
								{
									if(canceled){
										enablePushables();
										button_go.setEnabled(true);
										canceled = false;
										return;
									}
									W[j] += (double) Math.pow(cur[k]-X[j][k],2);
								}
								W[j] = (double) Math.exp(-1*W[j]/(2*sigma*sigma));
								D += W[j];
							}
							for(int j=0;j<DD;j++)
							{
								cur[j] = 0;
								for(int k=0;k<N;k++)
									cur[j] += W[k] * X[k][j] / D;
							}
						}
						else {
							double D = 0;
							int xi = (int)X[i][0];
							int yi = (int)X[i][1];
							int lower = Math.max(0,yi-(radius - 1));
							int upper = Math.min(height - 1,yi+(radius - 1));
							int left = Math.max(0,xi-(radius - 1));
							int right = Math.min(width - 1,xi+(radius - 1));
							double [] W = new double[(upper-lower+1)*(right-left+1)];
							for(int j = lower; j <= upper; j++)
							{
								for(int k = left; k <= right; k++)
								{
									int w_index = (j-lower) * (right - left+1) + (k-left);
									for(int l = 0; l < DD; l++)
									{
										if(canceled){
											enablePushables();
											button_go.setEnabled(true);
											canceled = false;
											return;
										}
										W[w_index] += (double)Math.pow(cur[l]-X[k+width*j][l],2);
									}
									W[w_index] = (double)Math.exp(-1*W[w_index]/(2*sigma*sigma));
									D += W[w_index];
								}
							}
							for(int j = 0 ; j < DD; j++)
							{
								cur[j] = 0;
								for(int l=lower; l <= upper; l++)
									for(int k = left; k <= right; k++)
									{
										int w_index = (l-lower) * (right-left+1) + (k-left);
										cur[j] = cur[j] + W[w_index] * X[k+width*l][j] / D;
									}
							}
						}

						int x = (int)cur[0]; //width
						int y = (int)cur[1]; //height
						int xi = (int)((cur[0] - x)*n); //within nxn [0,n)
						int yi = (int)((cur[1] - y)*n); //within nxn [0,n)
						//Do the updating of label table and current path
						/*
						 I check to see if cur is visited or not
						 */
						if(labeltable[x][y][xi][yi] != 0)
						{
							if(label != labeltable[x][y][xi][yi])
							{
								label--;
								int newlabel = labeltable[x][y][xi][yi];
								while(!path.isEmpty())
								{
									int [] temp = path.pop();
									labeltable[temp[0]]
									           [temp[1]]
									            [temp[2]]
									             [temp[3]]
									              = newlabel;
								}
								Labels[i] = newlabel;
								break iteration;
							}
						}
						else
						{
							int [] temp = new int[4];
							temp[0] = (int)x;
							temp[1] = (int)y;
							temp[2] = (int)xi;
							temp[3] = (int)yi;
							path.add(temp);
							labeltable[x][y][xi][yi] = label;
						}
						/*
						 if it is visited
							if it is not along the current path, I go along path and update the
						 indexes to the visited label and break
							if it is along the path I continue

						 if it is not visited, I update the path with the
						 current index and change the labeltable to the current
						 label
						 */

						//Updated, now we need to check for convergence

						double Xmov = 0;
						for(int j=0;j<DD;j++)
							Xmov+=Math.pow(prev[j]-cur[j],2);
						Xmov = (double) Math.sqrt(Xmov);
						convcrit = Xmov > tol;
					}
				if(Labels[i] == 0){
					Labels[i] = label;
					if(colors.size() == label-1)
						colors.add(cur);
				}
				label++;
				int[] rgb = new int[DD>5?4:3];
				if(cur.length > 3){
					double[] color = new double[3];
					double[] temp = colors.get(Labels[i]-1);
					for(int j = 0; j < 3; j++)
						color[j] = temp[j+2];
					color[0] = (color[0]/scale*103.5955f);
					color[1] = (color[1]/scale*(141.383f+229.2857f)-141.383f);
					color[2] = (color[2]/scale*(130.094f+126.5385f)-130.094f);
					CreateImage.luv2rgb(color[0], color[1], color[2], color);
					for(int j = 0; j < color.length; j++)
						rgb[j] = (int)color[j];
					if(DD>5)
						rgb[3] = 255;
				}
				else{
					double[] temp = colors.get(Labels[i]-1);
					for(int j = 0; j < 3; j++)
						rgb[j] = (int) (temp[2]/scale*255);
				}
				imgpanel_replay.setPixelColor((int)X[i][0], (int)X[i][1], rgb);
				imgpanel_replay.repaint();
			}

			K = label - 1;

			//Place it in
			storage_output.remove(imgpanel_output);
			imgpanel_output = new ImagePanel(
					new CreateImage(
							Labels,
							imgpanel_input.getMSWidth(),
							imgpanel_input.getMSHeight()
					).getImage(),1
			);
			imgpanel_output.validate();
			storage_output.add(imgpanel_output);
			app.validate();

			viewIts = 0;
			info_input.setText(inputString());
			info_param.setText(paramString());
			info_output.setText(outputString(1));
			app.validate();

			/** Re-enable buttons and menu options **/
			enablePushables();
		}
	}
}
