//
//  CreateImage.java
//  
//
//  Created by David King on 3/25/10.
//  Copyright 2010 Miguel A. Carreira-Perpinan. All rights reserved.
//

import java.awt.image.*;

public class CreateImage
{
	private BufferedImage img;
	private WritableRaster raster;
	public static int [][] color = {{255,0,0},
							  {0,255,0},
							  {0,0,255},
							  {255,255,0},
							  {255,0,255},
							  {0,255,255},
							  {128,128,128},
							  {255,255,255},
							  {0,0,0},
							  {128,0,128},
							  {128,128,0},
							  {0,128,128},
							  {0,128,0},
							  {128,0,0},
							  {0,0,128},
							  {128,128,255},
							  {128,255,128},
							  {255,128,128},
							  {255,255,128},
							  {255,128,255},
							  {128,255,255},
							  {128,255,0},
							  {128,0,255},
							  {255,0,128},
							  {255,128,0},
							  {0,255,128},
							  {0,128,255}};
	
	//This does not depend on what the color values are so
	//we do not need to convert
	public CreateImage(concomp c,double [][] X,int w,int h)
	{
		img = new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
		raster = (WritableRaster) img.getData();
		for(int i=0;i<c.compl.length;i++)
			raster.setPixel((int)X[i][0],(int)X[i][1],color[c.compl[i]%color.length]);
		img.setData(raster);
	}
	
	//This is for ms1
	public CreateImage(int [] Labels,int w,int h)
	{
		img = new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
		raster = (WritableRaster) img.getData();
		int x,y,N = Labels.length;
		for(int i=0;i<N;i++)
		{
			x = i%h;
			y = i/h;
			raster.setPixel(y,x,color[Labels[i]%color.length]);
		}
		img.setData(raster);
	}
	
	public CreateImage(double [][] X,int w,int h,double scale)
	{
		if(X[0].length >= 5)
			img = new BufferedImage(w,h,BufferedImage.TYPE_INT_RGB);
		else
			img = new BufferedImage(w,h,BufferedImage.TYPE_BYTE_GRAY);
		raster = (WritableRaster) img.getData();
		double [] temp = new double[3];
		double L,U,V;
		for(int index=0;index<X.length;index++)
		{
			if(X[0].length >= 5)
			{
				//First we scale the values back from scale and from [0,1]
				L = X[index][2]/scale*103.5955f;
				U = X[index][3]/scale*(141.383f+229.2857f)-141.383f;
				V = X[index][4]/scale*(130.094f+126.5385f)-130.094f;
				luv2rgb(L,U,V,temp);
				raster.setPixel((int)X[index][0],(int)X[index][1],temp);
			}
			else
			{
				temp[0] = X[index][2]/scale*255.0f;
				raster.setPixel((int)X[index][0],(int)X[index][1],temp);
			}
		}
		img.setData(raster);
	}
	
	
	public static void luv2rgb(double L, double U, double V, double []rgb) {

		//This is the case for black
		if(L == U && U == V && V == 0.0f)
		{
			rgb[0] = 0.0f;
			rgb[1] = 0.0f;
			rgb[2] = 0.0f;
			return;
		}
		
		//http://www.brucelindbloom.com
		
		// Standard illuminant C
		double x = 0.980708;
		double y = 1.0;
		double z = 1.182163;
		
		double u = (4*x) / (x + 15* y + 3 * z);
		double v = (9 * y) / (x + 15 * y + 3 * z);
		
		double eps = 0.008856f;
		double k = 903.3f;
		
		double Y = (double) (L>k*eps?Math.pow(((L+16)/116), 3):(L/k));
		
		double a = (double) ((1.0/3.0) * ((52 * L)/(U + 13 * L * u) - 1));
		double b = -5 * Y;
		double c = (double) (-1.0/3.0);
		double d = Y*((39*L)/(V+13*L*v) - 5);
		
		double X = (d - b) / (a - c);
		double Z = X * a + b;
		
		double r  = X* 1.762  + Y*-0.493 + Z*-0.211;
		double g  = X*-0.829  + Y* 1.688 + Z*-0.019;
		double bb = X* 0.085  + Y*-0.173 + Z* 1.037;
		
		double R = (r>.0031308f ? (double)Math.pow(r,1.0f/2.4f)*1.055f-0.055f:12.92f*r);
		double G = (g>.0031308f ? (double)Math.pow(g,1.0f/2.4f)*1.055f-0.055f:12.92f*g);
		double B = (bb>.0031308f ? (double)Math.pow(bb,1.0f/2.4f)*1.055f-0.055f:12.92f*bb);
		
		R*=255.0f;
		G*=255.0f;
		B*=255.0f;
		rgb[0] = R;
		rgb[1] = G;
		rgb[2] = B;
	}
	
	public BufferedImage getImage()
	{
		return img;
	};
}
