//
//  LoadImage.java
//  
//
//  Created by David King on 3/25/10.
//	Edited by Mark Crompton and Jimmy Yih on 5/17/11
//  Copyright 2010 Miguel A. Carreira-Perpinan. All rights reserved.
//

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.awt.image.IndexColorModel;
import java.awt.image.Raster;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

public class ImagePanel extends JPanel{
	private static final long serialVersionUID = 1L;
	//This is our globla image
	private BufferedImage img;
	private double zoom;
	WritableRaster wr;

	//This pains our Component with the graphics given (our image)
	public void paint(Graphics g)
	{
		if(img != null){
			int width = (int) (img.getWidth()*zoom);
			int height = (int) (img.getHeight()*zoom);
			g.drawImage(img,0,0,width,height,null);
		}
		else
			super.paint(g);
	}

	//Default Constructor
	public ImagePanel()
	{
		img = null;
		zoom = 1;
	}

	//This constructor takes in a file and creates the desired image
	public ImagePanel(File inputfile, double z)
	{
		try {
			img = ImageIO.read(inputfile);
			zoom = z;
			if(img.getType() == BufferedImage.TYPE_BYTE_BINARY || img.getType() == BufferedImage.TYPE_BYTE_INDEXED)
				convertToRGB(img.getType());
			wr = (WritableRaster) img.getData();
		}
		catch (IOException e) {
		}
	}

	//This constructor takes in a BufferedImage
	public ImagePanel(BufferedImage input, double z)
	{
		img = input;
		zoom = z;
		if(img.getType() == BufferedImage.TYPE_BYTE_BINARY || img.getType() == BufferedImage.TYPE_BYTE_INDEXED)
			convertToRGB(img.getType());
		wr = (WritableRaster) img.getData();
	}
	
	public ImagePanel(File input, double z, boolean ms1){
		this(input, z);
		convertToRGB(img.getType());
		wr = (WritableRaster) img.getData();
	}

	//This sets our preferredsize of our component
	public Dimension getPreferredSize()
	{
		if(img == null)
			return new Dimension(256,256);
		else
			return new Dimension((int)(img.getWidth(null)*zoom),(int)(img.getHeight(null)*zoom));
	}

	//We can get our image from this
	public BufferedImage getImage()
	{
		return img;
	}

	public void setPixelColor(int x, int y, int[] color){
		wr.setPixel(x, y, color);
		img.setData(wr);
	}
	
	
	public void convertToRGB(int type){
		switch(type){
		case BufferedImage.TYPE_BYTE_GRAY :
			WritableRaster originalRaster = (WritableRaster) img.getData();
			img = new BufferedImage(img.getData().getWidth(), img.getData().getHeight(), BufferedImage.TYPE_INT_RGB);
			wr = (WritableRaster) img.getData();
			for(int xx = 0; xx < img.getWidth(); xx++){
				for(int yy = 0; yy < img.getHeight(); yy++){
					int[] colorC = new int[3];
					colorC = originalRaster.getPixel(xx, yy, colorC);
					colorC[1] = colorC[0];
					colorC[2] = colorC[0];
					wr.setPixel(xx, yy, colorC);
				}
			}
			img.setData(wr);
			break;
		case BufferedImage.TYPE_BYTE_BINARY : 
			IndexColorModel icm = (IndexColorModel) img.getColorModel();
			Raster r = img.getData();
			img = icm.convertToIntDiscrete(r, false);
			break;
		case BufferedImage.TYPE_BYTE_INDEXED :
			BufferedImage rgb = new BufferedImage(img.getData().getWidth(), img.getData().getHeight(), BufferedImage.TYPE_INT_RGB);
			Graphics2D g = rgb.createGraphics();
			g.drawImage(img, null, 0, 0);
			img = rgb;
			break;
		}
	}
	
	//This returns the Pixels of the image in a NxD double array
	//Where N is the number of points and D is the dimension of each point
	public double [][] getMSPixels()
	{
		Raster r = img.getData();
		double[] temp = r.getPixel(0,0,(double []) null);
		int D = temp.length + 2;
		int width = r.getWidth();
		int height = r.getHeight();
		double [][] X = new double[width*height][D];
		for(int i=0;i<width;i++)
			for(int j=0;j<height;j++)
			{
				X[i*height + j][0]=i;
				X[i*height + j][1]=j;
				temp = r.getPixel(i,j,(double[])null);
				for(int k=0;k<D-2;k++)
				{
					X[i*height + j][k+2] = temp[k];
				}
			}
		return X;
	}

	public double [][] getMSPixels(double scale)
	{
		Raster r = img.getData();
		double[] temp = r.getPixel(0,0,(double []) null);
		int D = temp.length + 2;
		int width = r.getWidth();
		int height = r.getHeight();
		double [][] X = new double[width*height][D];
		//The values and procedure are taken from:
		double U,V,L;
		for(int j=0;j<width;j++)
			for(int k=0;k<height;k++)
			{
				int index = j*height + k;
				X[index][0]=j;
				X[index][1]=k;
				temp = r.getPixel(j,k,(double[])null);
				if(D >= 5)
				{
					rgb2luv(temp[0],temp[1],temp[2],temp);
					//We then scale from LUV to [0,1] to scale
					L = temp[0]/103.5955f*scale;
					U = (temp[1]+141.383f)/(141.383f+229.2857f)*scale;
					V = (temp[2]+130.094f)/(130.094f+126.5385f)*scale;
					X[index][2] = L;
					X[index][3] = U;
					X[index][4] = V;
					if(D == 6)
						X[index][5] = X[index][5] / 255 * scale;
				}
				else
					X[index][2] = temp[0]/255.0f*scale;
			}
		return X;
	}

	public void rgb2luv(double R, double G, double B, double []luv) {

		//This is the case for black
		if(R == G && G == B && B == 0.0f)
		{
			luv[0] = 0.0f;
			luv[1] = 0.0f;
			luv[2] = 0.0f;
			return;
		}

		//http://www.brucelindbloom.com

		double r, g, b, X, Y, Z, yr;
		double L;
		double eps = 216.f/24389.f;
		double k = 24389.f/27.f;

		// Standard illuminant C
		double Xr = 0.980708;
		double Yr = 1.0;
		double Zr = 1.182163;

		// RGB to XYZ

		r = R/255.f; //R 0..1
		g = G/255.f; //G 0..1
		b = B/255.f; //B 0..1

		// assuming sRGB (D65)
		if (r <= 0.04045)
			r = r/12.92;
		else
			r = (double) Math.pow((r+0.055)/1.055,2.4);

		if (g <= 0.04045)
			g = g/12.92;
		else
			g = (double) Math.pow((g+0.055)/1.055,2.4);

		if (b <= 0.04045)
			b = b/12.92;
		else
			b = (double) Math.pow((b+0.055)/1.055,2.4);


		X = 0.6578*r + 0.2062*g + 0.1375*b;
		Y = 0.3229*r + 0.6947*g + 0.0783*b;
		Z = 0.0991*g + 0.9664*b;

		// XYZ to Luv

		double u, v, u_, v_, ur_, vr_;

		u_ = 4*X / (X + 15*Y + 3*Z);
		v_ = 9*Y / (X + 15*Y + 3*Z);

		ur_ = 4*Xr / (Xr + 15*Yr + 3*Zr);
		vr_ = 9*Yr / (Xr + 15*Yr + 3*Zr);

		yr = Y/Yr;

		if ( yr > eps )
			L =  (double) (116*Math.pow(yr, 1/3.) - 16);
		else
			L = k * yr;

		u = 13*L*(u_ -ur_);
		v = 13*L*(v_ -vr_);

		luv[0] = L;
		luv[1] = u;
		luv[2] = v;      
	}

	public int getMSWidth()
	{
		return img.getData().getWidth();
	}

	public int getMSHeight()
	{
		return img.getData().getHeight();
	}

	//We can set our image from this
	public void setImage(BufferedImage input)
	{
		img = input;
		validate();
	}
}
